pub struct NeighborProvider<I: VectorId + IntoUsize> { /* private fields */ }Implementations
Sourceimpl<I: VectorId + IntoUsize> NeighborProvider<I>
impl<I: VectorId + IntoUsize> NeighborProvider<I>
Sourcepub fn new_with_config(max_degree: u32, config: Config) -> ANNResult<Self>
pub fn new_with_config(max_degree: u32, config: Config) -> ANNResult<Self>
Create a new instance based on bf-tree Config directly.
Sourcepub fn max_degree(&self) -> u32
pub fn max_degree(&self) -> u32
Return the maximum degree (number of neighbors per vector)
Sourcepub fn get_neighbors(
&self,
vector_id: I,
neighbors: &mut AdjacencyList<I>,
) -> ANNResult<()>
pub fn get_neighbors( &self, vector_id: I, neighbors: &mut AdjacencyList<I>, ) -> ANNResult<()>
Retrieve the neighbor list of a vector.
Does not acquire any lock. Callers must ensure appropriate synchronization.
Sourcepub fn set_neighbors(
&self,
vector_id: I,
neighbors: &[I],
buf: &mut [I],
) -> ANNResult<()>
pub fn set_neighbors( &self, vector_id: I, neighbors: &[I], buf: &mut [I], ) -> ANNResult<()>
Insert a neighbor list of a vector in bf-tree as a (K, V) pair.
Does not acquire any lock. Callers must ensure appropriate synchronization.
The list of neighbors and their length is written into the passed buffer
buf before writing the leading self.dim * std::mem::size_of::<I>()
bytes of it into the bf-tree.
Errors
- Neighbor length is larger than
self.max_degree() - Buffer length (in
Icells) is smaller thanmax_degree() + 1
Sourcepub fn append_vector(
&self,
vector_id: I,
new_neighbor_ids: &[I],
buf: &mut [I],
) -> ANNResult<()>
pub fn append_vector( &self, vector_id: I, new_neighbor_ids: &[I], buf: &mut [I], ) -> ANNResult<()>
Append unique vectors into a neighbor list.
Does not acquire any lock. Callers must ensure appropriate synchronization for the entire read-modify-write cycle.
The newly appended neighbor list will always be extended to ‘dim’ long to avoid frequent mem copy in bf-tree.
Note: assuming all neighbors in the input list, ‘new_neighbor_ids’, are valid
pub fn delete_vector(&self, vector_id: I) -> ANNResult<()>