pub struct Hidden(/* private fields */);Expand description
A struct with a private member that cannot be constructed outside of this module.
This is used to prevent users from calling internal methods directly.