CreateQuantProvider

Trait CreateQuantProvider 

pub trait CreateQuantProvider {
    type Target;

    // Required method
    fn create(self, bf_tree_config: Config) -> ANNResult<Self::Target>;
}
A helper trait to select the quant vector store.

This is also implemented for NoStore, which explicitly disables deletion related functionality

Required Associated Types

Required Methods

fn create(self, bf_tree_config: Config) -> ANNResult<Self::Target>

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementations on Foreign Types

Source

impl CreateQuantProvider for Poly<dyn Quantizer>

Allow a FixedChunkPQTable to be promoted to full quant vector store.

Source

type Target = QuantVectorProvider

Source

fn create(self, bf_tree_config: Config) -> ANNResult<Self::Target>

Implementors