pub struct VectorUnavailable {
pub id: usize,
pub err: VectorError,
}Fields
id: usizeerr: VectorErrorTrait Implementations
Sourcefn acknowledge<D>(self, _why: D)where
D: Display,
fn acknowledge<D>(self, _why: D)where
D: Display,
Consume self, acknowledging the transient error but proceeding with program logic. Read more
Sourcefn escalate<D>(self, why: D) -> ANNErrorwhere
D: Display,
fn escalate<D>(self, why: D) -> ANNErrorwhere
D: Display,
Report to
self that transient errors are not acceptable in the current context
and that the transient error is being upgraded to a full error. Read moreSourcefn acknowledge_with<F, D>(self, why: F)
fn acknowledge_with<F, D>(self, why: F)
Consume self, acknowledging the transient error but proceeding with program logic. Read more