AccessError

Type Alias AccessError 

pub type AccessError = RankedError<VectorUnavailable, ANNError>;

Aliased Type

pub enum AccessError {
    Transient(VectorUnavailable),
    Error(ANNError),
}

Variants



Transient(VectorUnavailable)

This error can be ignored if the allowed by the caller code. Such errors should be acknowledged (see [TransientError::acknowledge] before being dropped, but aren’t necessarily required to be.



Error(ANNError)

This error cannot be ignored and should be propagated. The caller should never suppress values in this alternative.