BfTree-based data provider for DiskANN async indexes.
This crate provides a BfTree-backed implementation of the DiskANN
DataProvider trait, enabling indexes that can
transparently spill to disk for datasets larger than available memory.
Re-exports
pub use provider::AsVectorDtype;pub use provider::BfTreePaths;pub use provider::BfTreeProvider;pub use provider::BfTreeProviderParameters;pub use provider::CreateQuantProvider;pub use provider::FullAccessor;pub use provider::GraphParams;pub use provider::Hidden;pub use provider::QuantAccessor;pub use provider::StartPoint;pub use provider::VectorDtype;
Modules
- neighbors
- Bf-Tree neighbor list provider.
- provider
- quant
- Bf-Tree quant vector provider.
- vectors
- Bf-Tree vector provider
Structs
- Config
- Bf-tree configuration for advanced usage. Bf-tree is designed to work well on various workloads that you don’t have to change the default configuration. This configuration is more for advanced users who want to understand the different components of the system, rather than performance tuning.
- Config
Error - Wrapper around
bf_tree::ConfigErrorthat implementsstd::error::Error. - Context
Metrics - Metrics recorded by
DefaultContext. - Insert
Error - NoStore
- Vector
Unavailable