pub struct BfTreeProvider<T, Q = QuantVectorProvider>where
T: VectorRepr,{ /* private fields */ }Expand description
An Bf-tree based implementation of a DataProvider built around the idea of having up to
two layers of vector stores: a full-precision store and an optional quantized store.
This provider uses the identity mapping between external and internal vector IDs.
Type Parameters:
-
T: The primitive element type of the full-precision vector. This is typically some type likef32orhalf::f16. -
Q: The full type of the quant vector store. This is not constrained by a trait and rather relies on implementation for several concrete types, including:QuantVectorProvider: A Bf-Tree based spherical quantized vector store.NoStore: Disable quantization altogether. Note that this disables all methods reached through quantization based [Accessor]s at compile-time.
Indexing Strategies
-
FullPrecision: Only retrieves data from the full-precision portion of the index. No quantized vectors are used. During search, start points are filtered from the final results. -
Quantized: Performs all operations (search, pruning, insert) entirely in the quantized space using spherical distance functions. Post-processing copies candidate IDs forward without reranking. Fastest option — full-precision vectors are not touched at query time.
Examples
The following code demonstrates how to instantiate and use the BfTreeProvider in
a number of different scenarios.
Full-Precision Only - No Deletes
This example demonstrates how to create a BfTreeProvider that only supports
full-precision vectors.
use diskann_bftree::provider::{
BfTreeProvider, BfTreeProviderParameters
};
use diskann_bftree::NoStore;
use diskann_vector::distance::Metric;
use diskann_utils::views::{Init, Matrix};
use bf_tree::Config;
use std::num::NonZeroUsize;
let parameters = BfTreeProviderParameters {
max_points: 5,
num_start_points: NonZeroUsize::new(1).unwrap(),
dim: 4,
metric: Metric::L2,
max_degree: 32,
vector_provider_config: Config::default(),
quant_vector_provider_config: Config::default(),
neighbor_list_provider_config: Config::default(),
graph_params: None,
use_snapshot: false,
};
// Create a table that supports 5 points and 1 start point.
let start_points = Matrix::new(Init(|| 0.0f32), 1, 4);
let provider = BfTreeProvider::<f32, _>::new(
parameters,
start_points.as_view(),
NoStore,
);Full-Precision and Spherical Quantization
To create a two-level provider with a spherical quantization-based quant vector store,
a Poly<dyn Quantizer> can be supplied for the quant_precursor argument, as this
implements the CreateQuantProvider trait.
use diskann_quantization::{
alloc::{GlobalAllocator, Poly, poly},
algorithms::TransformKind,
spherical::{iface, SphericalQuantizer, SupportedMetric, PreScale},
};
use diskann_utils::views::{Init, Matrix};
use diskann_bftree::provider::{
BfTreeProvider, BfTreeProviderParameters
};
use diskann_vector::distance::Metric;
use bf_tree::Config;
use std::num::NonZeroUsize;
use rand::rngs::StdRng;
use rand::SeedableRng;
let dim = 4;
let data = Matrix::new(Init(|| 1.0f32), 4, dim);
let mut rng = StdRng::seed_from_u64(42);
let sq = SphericalQuantizer::train(
data.as_view(), TransformKind::Null,
SupportedMetric::SquaredL2, PreScale::None,
&mut rng, GlobalAllocator,
).unwrap();
let imp = iface::Impl::<1>::new(sq).unwrap();
let poly = Poly::new(imp, GlobalAllocator).unwrap();
let quantizer: Poly<dyn iface::Quantizer> = poly!(iface::Quantizer, poly);
let parameters = BfTreeProviderParameters {
max_points: 5,
num_start_points: NonZeroUsize::new(1).unwrap(),
dim: 4,
metric: Metric::L2,
max_degree: 32,
vector_provider_config: Config::default(),
quant_vector_provider_config: Config::default(),
neighbor_list_provider_config: Config::default(),
graph_params: None,
use_snapshot: false,
};
// Create a table that supports 5 points and 1 start point.
let start_points = Matrix::new(Init(|| 0.0f32), 1, 4);
let provider = BfTreeProvider::<f32, _>::new(
parameters,
start_points.as_view(),
quantizer,
);Implementations
Sourceimpl<T, Q> BfTreeProvider<T, Q>where
T: VectorRepr,
impl<T, Q> BfTreeProvider<T, Q>where
T: VectorRepr,
Sourcepub fn new<TQ>(
params: BfTreeProviderParameters,
start_points: MatrixView<'_, T>,
quant_precursor: TQ,
) -> ANNResult<Self>where
Self: StartPoint<T>,
TQ: CreateQuantProvider<Target = Q>,
pub fn new<TQ>(
params: BfTreeProviderParameters,
start_points: MatrixView<'_, T>,
quant_precursor: TQ,
) -> ANNResult<Self>where
Self: StartPoint<T>,
TQ: CreateQuantProvider<Target = Q>,
Construct a new data provider with start points initialized.
This is the primary constructor for BfTreeProvider where start points are known from the
beginning. It creates the provider and sets the start points in one operation.
Arguments
params: An instance ofBfTreeProviderParameterscollecting shared configuration information.start_points: A matrix view containing the start point vectors. The number of rows must matchparams.num_start_points.get().quant_precursor: A precursor type for the quantizer layer.
Type Constraints
Self: StartPoint<T>- The provider must implement theStartPointtrait.
Sourcepub fn starting_points(&self) -> ANNResult<Vec<u32>>
pub fn starting_points(&self) -> ANNResult<Vec<u32>>
Return a vector of starting points.
Sourcepub fn iter(&self) -> Range<u32>
pub fn iter(&self) -> Range<u32>
An iterator over all ids including start points (even if they are deleted).
pub fn num_start_points(&self) -> usize
Sourcepub fn max_points(&self) -> usize
pub fn max_points(&self) -> usize
Return the maximum number of points (excluding frozen/start points)
Sourcepub fn max_degree(&self) -> u32
pub fn max_degree(&self) -> u32
Return the maximum degree from the neighbor provider
Sourceimpl<T> BfTreeProvider<T, QuantVectorProvider>where
T: VectorRepr,
impl<T> BfTreeProvider<T, QuantVectorProvider>where
T: VectorRepr,
Sourcepub fn counts_for_get_vector(&self) -> (usize, usize)
pub fn counts_for_get_vector(&self) -> (usize, usize)
Return the number of vector reads for full-precision and quant-vectors respectively
Sourceimpl<T> BfTreeProvider<T, NoStore>where
T: VectorRepr,
impl<T> BfTreeProvider<T, NoStore>where
T: VectorRepr,
Sourcepub fn counts_for_get_vector(&self) -> (usize, usize)
pub fn counts_for_get_vector(&self) -> (usize, usize)
Return the number of vector reads for full-precision and quant-vectors respectively
Sourceimpl<T, Q> BfTreeProvider<T, Q>where
T: VectorRepr,
Q: AsyncFriendly,
impl<T, Q> BfTreeProvider<T, Q>where
T: VectorRepr,
Q: AsyncFriendly,
pub fn neighbors(&self) -> &NeighborProvider<u32>