pub trait CreateQuantProvider {
type Target;
// Required method
fn create(self, bf_tree_config: Config) -> ANNResult<Self::Target>;
}Expand description
A helper trait to select the quant vector store.
This is also implemented for NoStore, which explicitly disables deletion
related functionality
Required Associated Types
Required Methods
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".
Implementations on Foreign Types
Sourceimpl CreateQuantProvider for Poly<dyn Quantizer>
Allow a FixedChunkPQTable to be promoted to full quant vector store.
impl CreateQuantProvider for Poly<dyn Quantizer>
Allow a FixedChunkPQTable to be promoted to full quant vector store.